@@ -9,11 +9,12 @@ import coverage
 import py
 import pytest
 import virtualenv
+import xdist
+from fields import Namespace
 from process_tests import TestProcess as _TestProcess
 from process_tests import dump_on_error
 from process_tests import wait_for_strings
 from six import exec_
-from fields import Namespace
 
 try:
     from StringIO import StringIO
@@ -28,10 +29,10 @@ SCRIPT = '''
 import sys, helper
 
 def pytest_generate_tests(metafunc):
-    for i in range(10):
-        metafunc.addcall()
+    for i in [10]:
+        metafunc.parametrize('p', range(i))
 
-def test_foo():
+def test_foo(p):
     x = True
     helper.do_stuff()  # get some coverage in some other completely different location
     if sys.version_info[0] > 5:
@@ -65,16 +66,17 @@ if idx == 1:
 '''
 
 SCRIPT_PARENT = '''
+import os
 import subprocess
 import sys
 
 def pytest_generate_tests(metafunc):
-    for i in range(2):
-        metafunc.addcall(funcargs=dict(idx=i))
+    for i in [2]:
+        metafunc.parametrize('idx', range(i))
 
 def test_foo(idx):
     out, err = subprocess.Popen(
-        [sys.executable, 'child_script.py', str(idx)],
+        [sys.executable, os.path.join(os.path.dirname(__file__), 'child_script.py'), str(idx)],
         stdout=subprocess.PIPE,
         stderr=subprocess.PIPE).communicate()
 
@@ -89,8 +91,8 @@ import sys
 import os
 
 def pytest_generate_tests(metafunc):
-    for i in range(2):
-        metafunc.addcall(funcargs=dict(idx=i))
+    for i in [2]:
+        metafunc.parametrize('idx', range(i))
 
 def test_foo(idx):
     os.mkdir("foobar")
@@ -113,8 +115,8 @@ import sys
 import os
 
 def pytest_generate_tests(metafunc):
-    for i in range(2):
-        metafunc.addcall(funcargs=dict(idx=i))
+    for i in [2]:
+        if metafunc.function is test_foo: metafunc.parametrize('idx', range(i))
 
 def test_foo(idx):
     os.mkdir("foobar")
@@ -134,7 +136,7 @@ SCRIPT_FUNCARG = '''
 import coverage
 
 def test_foo(cov):
-    assert isinstance(cov, coverage.coverage)
+    assert isinstance(cov, coverage.Coverage)
 '''
 
 SCRIPT_FUNCARG_NOT_ACTIVE = '''
@@ -142,18 +144,12 @@ def test_foo(cov):
     assert cov is None
 '''
 
-SCRIPT_FAIL = '''
-def test_fail():
-    assert False
-
-'''
-
 CHILD_SCRIPT_RESULT = '[56] * 100%'
-PARENT_SCRIPT_RESULT = '8 * 100%'
+PARENT_SCRIPT_RESULT = '9 * 100%'
 DEST_DIR = 'cov_dest'
 REPORT_NAME = 'cov.xml'
 
-xdist = pytest.mark.parametrize('opts', ['', '-n 1'], ids=['nodist', 'xdist'])
+xdist_params = pytest.mark.parametrize('opts', ['', '-n 1'], ids=['nodist', 'xdist'])
 
 
 @pytest.fixture(params=[
@@ -168,6 +164,7 @@ def prop(request):
         code2=SCRIPT2,
         conf=request.param[0],
         fullconf='[run]\n%s\n' % request.param[0],
+        prefixedfullconf='[coverage:run]\n%s\n' % request.param[0],
         args=request.param[1].split(),
         result=request.param[2],
         result2=request.param[3],
@@ -227,7 +224,7 @@ def test_annotate_output_dir(testdir):
     assert result.ret == 0
 
 
-def test_html_output_dir(testdir,prop):
+def test_html_output_dir(testdir, prop):
     script = testdir.makepyfile(SCRIPT)
 
     result = testdir.runpytest('-v',
@@ -271,10 +268,7 @@ def test_term_output_dir(testdir):
                                '--cov-report=term:' + DEST_DIR,
                                script)
 
-    # backport of argparse to py26 doesn't display ArgumentTypeError message
     result.stderr.fnmatch_lines([
-        '*argument --cov-report: *',
-    ] if tuple(sys.version_info[:2]) == (2, 6) else [
         '*argument --cov-report: output specifier not supported for: "term:%s"*' % DEST_DIR,
     ])
     assert result.ret != 0
@@ -288,10 +282,7 @@ def test_term_missing_output_dir(testdir):
                                '--cov-report=term-missing:' + DEST_DIR,
                                script)
 
-    # backport of argparse to py26 doesn't display ArgumentTypeError message
     result.stderr.fnmatch_lines([
-        '*argument --cov-report: *',
-    ] if tuple(sys.version_info[:2]) == (2, 6) else [
         '*argument --cov-report: output specifier not supported for: '
         '"term-missing:%s"*' % DEST_DIR,
     ])
@@ -400,6 +391,79 @@ def test_central_coveragerc(testdir, prop):
     assert result.ret == 0
 
 
+@xdist_params
+def test_central_with_path_aliasing(testdir, monkeypatch, opts, prop):
+    mod1 = testdir.mkdir('src').join('mod.py')
+    mod1.write(SCRIPT)
+    mod2 = testdir.mkdir('aliased').join('mod.py')
+    mod2.write(SCRIPT)
+    script = testdir.makepyfile('''
+from mod import *
+''')
+    testdir.tmpdir.join('setup.cfg').write("""
+[coverage:paths]
+source =
+    src
+    aliased
+[coverage:run]
+source = mod
+parallel = true
+%s
+""" % prop.conf)
+
+    monkeypatch.setitem(os.environ, 'PYTHONPATH', os.pathsep.join([os.environ.get('PYTHONPATH', ''), 'aliased']))
+    result = testdir.runpytest('-v', '-s',
+                               '--cov',
+                               '--cov-report=term-missing',
+                               script, *opts.split()+prop.args)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'src[\\/]mod* %s *' % prop.result,
+        '*10 passed*',
+    ])
+
+    # single-module coverage report
+    assert all(not line.startswith('TOTAL ') for line in result.stdout.lines[-4:])
+
+    assert result.ret == 0
+
+
+def test_subprocess_with_path_aliasing(testdir, monkeypatch):
+    src = testdir.mkdir('src')
+    src.join('parent_script.py').write(SCRIPT_PARENT)
+    src.join('child_script.py').write(SCRIPT_CHILD)
+    aliased = testdir.mkdir('aliased')
+    parent_script = aliased.join('parent_script.py')
+    parent_script.write(SCRIPT_PARENT)
+    aliased.join('child_script.py').write(SCRIPT_CHILD)
+
+    testdir.tmpdir.join('.coveragerc').write("""
+[paths]
+source =
+    src
+    aliased
+[run]
+source =
+    parent_script
+    child_script
+parallel = true
+""")
+
+    monkeypatch.setitem(os.environ, 'PYTHONPATH', os.pathsep.join([os.environ.get('PYTHONPATH',''), 'aliased']))
+    result = testdir.runpytest('-v',
+                               '--cov',
+                               '--cov-report=term-missing',
+                               parent_script)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'src[\\/]child_script* %s*' % CHILD_SCRIPT_RESULT,
+        'src[\\/]parent_script* %s*' % PARENT_SCRIPT_RESULT,
+    ])
+    assert result.ret == 0
+
+
 def test_show_missing_coveragerc(testdir, prop):
     script = testdir.makepyfile(prop.code)
     testdir.tmpdir.join('.coveragerc').write("""
@@ -430,7 +494,11 @@ show_missing = true
 
 
 def test_no_cov_on_fail(testdir):
-    script = testdir.makepyfile(SCRIPT_FAIL)
+    script = testdir.makepyfile('''
+def test_fail():
+    assert False
+
+''')
 
     result = testdir.runpytest('-v',
                                '--cov=%s' % script.dirpath(),
@@ -453,12 +521,16 @@ def test_no_cov(testdir):
                                script)
     result.stdout.fnmatch_lines_random([
         'WARNING: Coverage disabled via --no-cov switch!',
-        '*1 warnings*' if pytest.__version__.startswith('2.7') else 'WCOV-U1*Coverage disabled via --no-cov switch!',
+        '*Coverage disabled via --no-cov switch!',
     ])
 
 
 def test_cov_and_failure_report_on_fail(testdir):
-    script = testdir.makepyfile(SCRIPT + SCRIPT_FAIL)
+    script = testdir.makepyfile(SCRIPT + '''
+def test_fail(p):
+    assert False
+
+''')
 
     result = testdir.runpytest('-v',
                                '--cov=%s' % script.dirpath(),
@@ -524,7 +596,14 @@ def test_dist_not_collocated(testdir, prop):
     script = testdir.makepyfile(prop.code)
     dir1 = testdir.mkdir('dir1')
     dir2 = testdir.mkdir('dir2')
-    testdir.tmpdir.join('.coveragerc').write(prop.fullconf)
+    testdir.tmpdir.join('.coveragerc').write('''
+[run]
+%s
+[paths]
+source =
+    .
+    dir1
+    dir2''' % prop.conf)
 
     result = testdir.runpytest('-v',
                                '--cov=%s' % script.dirpath(),
@@ -545,6 +624,39 @@ def test_dist_not_collocated(testdir, prop):
     assert result.ret == 0
 
 
+def test_dist_not_collocated_coveragerc_source(testdir, prop):
+    script = testdir.makepyfile(prop.code)
+    dir1 = testdir.mkdir('dir1')
+    dir2 = testdir.mkdir('dir2')
+    testdir.tmpdir.join('.coveragerc').write('''
+[run]
+%s
+source = %s
+[paths]
+source =
+    .
+    dir1
+    dir2''' % (prop.conf, script.dirpath()))
+
+    result = testdir.runpytest('-v',
+                               '--cov',
+                               '--cov-report=term-missing',
+                               '--dist=load',
+                               '--tx=popen//chdir=%s' % dir1,
+                               '--tx=popen//chdir=%s' % dir2,
+                               '--rsyncdir=%s' % script.basename,
+                               '--rsyncdir=.coveragerc',
+                               '--max-slave-restart=0', '-s',
+                               script, *prop.args)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_dist_not_collocated* %s *' % prop.result,
+        '*10 passed*'
+    ])
+    assert result.ret == 0
+
+
 def test_central_subprocess(testdir):
     scripts = testdir.makepyfile(parent_script=SCRIPT_PARENT,
                                  child_script=SCRIPT_CHILD)
@@ -581,8 +693,8 @@ parallel = true
 
     result.stdout.fnmatch_lines([
         '*- coverage: platform *, python * -*',
-        'child_script* %s*' % CHILD_SCRIPT_RESULT,
-        'parent_script* 100%*',
+        '*child_script* %s*' % CHILD_SCRIPT_RESULT,
+        '*parent_script* 100%*',
     ])
     assert result.ret == 0
 
@@ -622,8 +734,6 @@ def test_foo():
     testdir.makefile('', coveragerc="""
 [run]
 parallel = true
-omit =
-    */__init__.py
 """)
     result = testdir.runpytest('-v',
                                '--cov-config=coveragerc',
@@ -666,15 +776,21 @@ def test_dist_subprocess_not_collocated(testdir, tmpdir):
 
     dir1 = tmpdir.mkdir('dir1')
     dir2 = tmpdir.mkdir('dir2')
-
+    testdir.tmpdir.join('.coveragerc').write('''
+[paths]
+source =
+    %s
+    */dir1
+    */dir2
+''' % scripts.dirpath())
     result = testdir.runpytest('-v',
                                '--cov=%s' % scripts.dirpath(),
-                               '--cov-report=term-missing',
                                '--dist=load',
                                '--tx=popen//chdir=%s' % dir1,
                                '--tx=popen//chdir=%s' % dir2,
                                '--rsyncdir=%s' % child_script,
                                '--rsyncdir=%s' % parent_script,
+                               '--rsyncdir=.coveragerc',
                                '--max-slave-restart=0',
                                parent_script)
 
@@ -688,7 +804,10 @@ def test_dist_subprocess_not_collocated(testdir, tmpdir):
 
 def test_invalid_coverage_source(testdir):
     script = testdir.makepyfile(SCRIPT)
-
+    testdir.makeini("""
+        [tool:pytest]
+        console_output_style=classic
+    """)
     result = testdir.runpytest('-v',
                                '--cov=non_existent_module',
                                '--cov-report=term-missing',
@@ -698,20 +817,14 @@ def test_invalid_coverage_source(testdir):
         '*10 passed*'
     ])
     result.stderr.fnmatch_lines([
-        'Coverage.py warning: No data was collected.'
+        'Coverage.py warning: No data was collected.*'
     ])
+    result.stdout.fnmatch_lines([
+        '*Failed to generate report: No data to report.',
+    ])
+    assert result.ret == 0
 
-    if StrictVersion(coverage.__version__) <= StrictVersion("3.8"):
-        # older `coverage report` doesn't error on missing imports
-        assert result.ret == 0
-    else:
-        # newer `coverage report` errors on missing importts
-        result.stderr.fnmatch_lines([
-            'ERROR: Failed to generate report: No data to report.',
-        ])
-        assert result.ret != 0
-
-    matching_lines = [line for line in result.outlines if '%' in line]
+    matching_lines = [line for line in result.outlines if '%' in line and 'PASSED' not in line]
     assert not matching_lines
 
 
@@ -725,10 +838,12 @@ def test_dist_missing_data(testdir):
         exe = os.path.join(venv_path, 'bin', 'python')
     subprocess.check_call([
         exe,
-        '-mpip' if sys.version_info >= (2, 7) else '-mpip.__main__',
+        '-mpip',
         'install',
         'py==%s' % py.__version__,
-        'pytest==%s' % pytest.__version__
+        'pytest==%s' % pytest.__version__,
+        'pytest_xdist==%s' % xdist.__version__
+
     ])
     script = testdir.makepyfile(SCRIPT)
 
@@ -803,6 +918,37 @@ def test_run_target():
     assert result.ret == 0
 
 
+def test_multiprocessing_subprocess_no_source(testdir):
+    py.test.importorskip('multiprocessing.util')
+
+    script = testdir.makepyfile('''
+import multiprocessing
+
+def target_fn():
+    a = True
+    return a
+
+def test_run_target():
+    p = multiprocessing.Process(target=target_fn)
+    p.start()
+    p.join()
+''')
+
+    result = testdir.runpytest('-v',
+                               '--cov',
+                               '--cov-report=term-missing',
+                               script)
+
+    result.stdout.fnmatch_lines([
+        '*- coverage: platform *, python * -*',
+        'test_multiprocessing_subprocess* 8 * 100%*',
+        '*1 passed*'
+    ])
+    assert result.ret == 0
+
+
+@pytest.mark.skipif('sys.platform == "win32"',
+                    reason="multiprocessing don't support clean process temination on Windows")
 def test_multiprocessing_subprocess_with_terminate(testdir):
     py.test.importorskip('multiprocessing.util')
 
@@ -822,6 +968,7 @@ def target_fn():
 def test_run_target():
     p = multiprocessing.Process(target=target_fn)
     p.start()
+    time.sleep(0.5)
     event.wait(1)
     p.terminate()
     p.join()
@@ -834,7 +981,7 @@ def test_run_target():
 
     result.stdout.fnmatch_lines([
         '*- coverage: platform *, python * -*',
-        'test_multiprocessing_subprocess* 15 * 100%*',
+        'test_multiprocessing_subprocess* 16 * 100%*',
         '*1 passed*'
     ])
     assert result.ret == 0
@@ -880,7 +1027,8 @@ def test_cover_looponfail(testdir, monkeypatch):
     testdir.makeconftest(CONFTEST)
     script = testdir.makepyfile(BASIC_TEST)
 
-    monkeypatch.setattr(testdir, 'run', lambda *args: _TestProcess(*map(str, args)))
+    monkeypatch.setattr(testdir, 'run',
+                        lambda *args, **kwargs: _TestProcess(*map(str, args)))
     with testdir.runpytest('-v',
                            '--cov=%s' % script.dirpath(),
                            '--looponfail',
@@ -908,6 +1056,46 @@ def test_cover_conftest_dist(testdir):
     result.stdout.fnmatch_lines([CONF_RESULT])
 
 
+def test_no_cover_marker(testdir):
+    testdir.makepyfile(mod=MODULE)
+    script = testdir.makepyfile('''
+import pytest
+import mod
+import subprocess
+import sys
+
+@pytest.mark.no_cover
+def test_basic():
+    mod.func()
+    subprocess.check_call([sys.executable, '-c', 'from mod import func; func()'])    
+''')
+    result = testdir.runpytest('-v', '-ra', '--strict',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+    assert result.ret == 0
+    result.stdout.fnmatch_lines(['mod* 2 * 1 * 50% * 2'])
+
+
+def test_no_cover_fixture(testdir):
+    testdir.makepyfile(mod=MODULE)
+    script = testdir.makepyfile('''
+import mod
+import subprocess
+import sys
+
+def test_basic(no_cover):
+    mod.func()
+    subprocess.check_call([sys.executable, '-c', 'from mod import func; func()'])    
+''')
+    result = testdir.runpytest('-v', '-ra', '--strict',
+                               '--cov=%s' % script.dirpath(),
+                               '--cov-report=term-missing',
+                               script)
+    assert result.ret == 0
+    result.stdout.fnmatch_lines(['mod* 2 * 1 * 50% * 2'])
+
+
 COVERAGERC = '''
 [report]
 # Regexes for lines to exclude from consideration
@@ -1069,7 +1257,10 @@ def test_dist_bare_cov(testdir):
 
 
 def test_not_started_plugin_does_not_fail(testdir):
-    plugin = pytest_cov.plugin.CovPlugin(None, None, start=False)
+    class ns:
+        cov_source = [True]
+        cov_report = ''
+    plugin = pytest_cov.plugin.CovPlugin(ns, None, start=False)
     plugin.pytest_runtestloop(None)
     plugin.pytest_terminal_summary(None)
 
@@ -1155,8 +1346,8 @@ def test_external_data_file_negative(testdir):
     assert glob.glob(str(testdir.tmpdir.join('.coverage*')))
 
 
-@xdist
-def xtest_append_coverage(testdir, opts, prop):
+@xdist_params
+def test_append_coverage(testdir, opts, prop):
     script = testdir.makepyfile(test_1=prop.code)
     testdir.tmpdir.join('.coveragerc').write(prop.fullconf)
     result = testdir.runpytest('-v',
@@ -1178,10 +1369,10 @@ def xtest_append_coverage(testdir, opts, prop):
     ])
 
 
-@xdist
-def xtest_do_not_append_coverage(testdir, opts, prop):
+@xdist_params
+def test_do_not_append_coverage(testdir, opts, prop):
     script = testdir.makepyfile(test_1=prop.code)
-    testdir.tmpdir.join('.coveragerc').write("")
+    testdir.tmpdir.join('.coveragerc').write(prop.fullconf)
     result = testdir.runpytest('-v',
                                '--cov=%s' % script.dirpath(),
                                script,
